@@ -23,10 +23,10 @@ along with this program.  If not, see
 --]]
 
 arbmarks = {
-   {a="bismillah", b="^^^^fdfd", c="\\arb[trans]{bi-ismi \\uc{'l-l_ahi} 'l-ra.hm_ani 'l-ra.hImi}"},
-   {a="salam", b="^^^^fdf5", c="\\arb[trans]{.sall_A\\arbnull{'l-l_ahu} \\uc{'l-l_ahu} `alay-hi wa-sallama}"},
-   {a="slm", b="^^^^fdfa", c="\\arb[trans]{.sall_A\\arbnull{'l-l_ahu} \\uc{'l-l_ahu} `alay-hi wa-sallama}"},
-   {a="jalla", b="^^^^fdfb", c="\\arb[trans]{^galla ^galAla-hu}"}
+   {a="bismillah", b="\\arabicfont{}^^^^fdfd", c="\\arb[trans]{bi-ismi \\uc{'l-l_ahi} 'l-ra.hm_ani 'l-ra.hImi}"},
+   {a="salam", b="\\arabicfont{}^^^^fdf5", c="\\arb[trans]{.sall_A\\arbnull{'l-l_ahu} \\uc{'l-l_ahu} `alay-hi wa-sallama}"},
+   {a="jalla", b="\\arabicfont{}^^^^fdfb", c="\\arb[trans]{^galla ^galAla-hu}"},
+   {a="slm", b="\\arabicfont{}^^^^fdfa", c="\\arb[trans]{.sall_A\\arbnull{'l-l_ahu} \\uc{'l-l_ahu} `alay-hi wa-sallama}"}
 }
 
 abjad = {
@@ -168,7 +168,7 @@ buckwalter = {
    {a="F", b="aN"},
    {a="K", b="iN"},
    {a="o", b="\""},
-   {a="P", b="o"}, -- pass on to \arbnull
+   {a="P", b="O"}, -- pass on to \arbnull
    -- hard-coded connective alif
    {a="%[", b="ٱ"},
    {a="L", b="ٱ"} -- BW safe
@@ -179,9 +179,9 @@ hamza = {
    {a="(')(%-)([uaiUAI])", b="%1%3"},
    -- next lines for ʾalif alone
    {a="(%.A)([uai]?)l%-(%^n)", b="ا%2ل%3"}, --additional (^n is lunar)
-   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
+   {a="([%(%[%|%<%s%-O])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
    {a="(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="ا%2ل%3%3"},
-   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4%4"}, --p
+   {a="([%(%[%|%<%s%-O])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4%4"}, --p
    {a="%.A", b="ا"},
    -- hard coded hamza
    {a="|\"'", b="ء"},
@@ -211,7 +211,9 @@ hamza = {
    {a="'a'([^uaiUAI])", b="آ%1"},
    {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
+   {a="\'a?A(O[%_%^%.]?[%`%'btjghxdrzsfqklmnywAY]-O)", b="آ"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
+   {a="(A)(O%'[%S]-O)", b="آ"},
    {a="(A)(')(uN?%p*)$", b="aآء%3"},
    {a="(A)(')(uN?)(%p*%s)", b="aآء%3%4"},
    {a="(A)(')(iN?%p*)$", b="aآء%3"},
@@ -221,10 +223,10 @@ hamza = {
    {a="(A)(')", b="aآء"}, -- historic madda
    -- initial (needs both ^ and %W patterns)
    -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
-   {a="^('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
-   {a="(%W)('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
-   {a="^('aw)(o)(\"?[uai])([%S]-o)", b="%1i"},
-   {a="(%W)('aw)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
+   {a="(%W)('aw)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1%2i"},
+   {a="^('aw)(O)(\"?[uai])([%S]-O)", b="%1i"},
+   {a="(%W)('aw)(O)(\"?[uai])([%S]-O)", b="%1%2i"},
    {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
    {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
@@ -296,9 +298,9 @@ hamzaeasy = { -- differences marked below with 'easy'
    {a="(')(%-)([uaiUAI])", b="%1%3"},
    -- next lines for ʾalif alone (easy)
    {a="(%.A)([uai]?)l%-(%^n)", b="ا%2ل%3"}, --additional (^n is lunar)
-   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
+   {a="([%(%[%|%<%s%-O])(%.A)([uai]?)l%-(%^n)", b="%1ا%3%4"}, --additional (^n is lunar) --p
    {a="(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="ا%2ل%3"},
-   {a="([%(%[%|%<%s%-o])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4"}, --p
+   {a="([%(%[%|%<%s%-O])(%.A)([uai]?)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%3ل%4"}, --p
    {a="%.A", b="ا"},
    -- hard coded hamza
    {a="|\"'", b="ء"},
@@ -328,8 +330,10 @@ hamzaeasy = { -- differences marked below with 'easy'
    {a="'a'([^uaiUAI])", b="آ%1"},
    {a="([^uiyUI])\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    {a="^\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="آ%1"},
+   {a="\'a?A(O[%_%^%.]?[%`%'btjghxdrzsfqklmnywAY]-O)", b="آ"},
    {a="(%W)\'a?A([%_%^%.]?[%`%'btjghxdrzsfqklmnywAY])", b="%1آ%2"},
    --easy (begin)
+   {a="(A)(O%'[%S]-O)", b="ا"},
    {a="(A)(')(uN?%p*)$", b="aاء%3"},
    {a="(A)(')(uN?)(%p*%s)", b="aاء%3%4"},
    {a="(A)(')(iN?%p*)$", b="aاء%3"},
@@ -340,10 +344,10 @@ hamzaeasy = { -- differences marked below with 'easy'
    --easy (end)
    -- initial (needs both ^ and %W patterns)
    -- 'aw: the diphthong is to be resolved into 'awi' (next 8 lines)
-   {a="^('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
-   {a="(%W)('aw)(o)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
-   {a="^('aw)(o)(\"?[uai])([%S]-o)", b="%1i"},
-   {a="(%W)('aw)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^('aw)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
+   {a="(%W)('aw)(O)('[%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1%2i"},
+   {a="^('aw)(O)(\"?[uai])([%S]-O)", b="%1i"},
+   {a="(%W)('aw)(O)(\"?[uai])([%S]-O)", b="%1%2i"},
    {a="^('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    {a="(%W)('aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
    {a="^('aw)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"},
@@ -422,17 +426,17 @@ hamzaeasy = { -- differences marked below with 'easy'
 
 tanwin = {
    -- assimilations (begin)
-   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([rlmnwy])", b="%4%4"},
+   {a="(O[%S]-)(%-?[uai]N[UI]?)(O)([rlmnwy])", b="%4%4"},
    {a="(%-?[uai]NU)(%s)([rlmnwy])", b="%1%2%3%3"},
    -- assimilations (end)
-   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([uai])", b="%4"},
+   {a="(O[%S]-)(%-?[uai]N[UI]?)(O)([uai])", b="%4"},
    {a="%-?uNU", b="ٌو"},
    {a="%-?aNU", b="ًوا"},
    {a="%-?iNU", b="ٍو"},
    -- assimilations (begin)
    {a="%-?(uN)(%s)([rlmnwy])", b="ٌ%2%3%3"},
-   {a="(o[%S]-)(%-?aN)(_A)(o)([rlmnwy])", b="%5%5"},
-   {a="(o[%S]-)(%-?aN)(Y)(o)([rlmnwy])", b="%5%5"},
+   {a="(O[%S]-)(%-?aN)(_A)(O)([rlmnwy])", b="%5%5"},
+   {a="(O[%S]-)(%-?aN)(Y)(O)([rlmnwy])", b="%5%5"},
    {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
    {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
@@ -440,8 +444,8 @@ tanwin = {
    {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
    {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
-   {a="(o[%S]-)(%-?aN)(_A)(o)([uai])", b="%5"},
-   {a="(o[%S]-)(%-?aN)(Y)(o)([uai])", b="%5"},
+   {a="(O[%S]-)(%-?aN)(_A)(O)([uai])", b="%5"},
+   {a="(O[%S]-)(%-?aN)(Y)(O)([uai])", b="%5"},
    -- quoted tanwīn (begin)
    {a="%-?(\"uN)", b=""},
    {a="(B)%-?(\"aN)", b="%1"},
@@ -454,6 +458,13 @@ tanwin = {
    -- quoted tanwīn (end)
    {a="%-?(uN)", b="ٌ"},
    {a="(B)%-?(aN)", b="%1ً"},
+   -- needed by \arbcolor:
+   {a="%-?(aN)(O[%S]-%_AO)", b="ً"},
+   {a="%-?(aN)(O[%S]-YO)", b="ً"},
+   {a="(O[%S]-TO)%-?(aN)", b="ً"},
+   {a="(O[%S]-)([اآ])(ء)(O)%-?(aN)", b="ً"}, --new
+   {a="(O[%S]-[^TA]O)%-?(aN)", b="ًا"},
+   --
    {a="%-?(aN)(_A)", b="ًى"},
    {a="%-?(aN)(Y)", b="ًى"},
    {a="(T)%-?(aN)", b="%1ً"},
@@ -464,17 +475,17 @@ tanwin = {
 
 tanwineasy = { -- 'easy' requires some lines to be taken out:
    -- assimilations (begin)
---   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([rlmnwy])", b="%4%4"},
+--   {a="(O[%S]-)(%-?[uai]N[UI]?)(O)([rlmnwy])", b="%4%4"},
 --   {a="(%-?[uai]NU)(%s)([rlmnwy])", b="%1%2%3%3"},
    -- assimilations (end)
-   {a="(o[%S]-)(%-?[uai]N[UI]?)(o)([uai])", b="%4"},
+   {a="(O[%S]-)(%-?[uai]N[UI]?)(O)([uai])", b="%4"},
    {a="%-?uNU", b="ٌو"},
    {a="%-?aNU", b="ًوا"},
    {a="%-?iNU", b="ٍو"},
    -- assimilations (begin)
 --   {a="%-?(uN)(%s)([rlmnwy])", b="ٌ%2%3%3"},
---   {a="(o[%S]-)(%-?aN)(_A)(o)([rlmnwy])", b="%5%5"},
---   {a="(o[%S]-)(%-?aN)(Y)(o)([rlmnwy])", b="%5%5"},
+--   {a="(O[%S]-)(%-?aN)(_A)(O)([rlmnwy])", b="%5%5"},
+--   {a="(O[%S]-)(%-?aN)(Y)(O)([rlmnwy])", b="%5%5"},
 --   {a="%-?(aN)(_A)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --   {a="%-?(aN)(Y)(%s)([rlmnwy])", b="ًى%3%4%4"},
 --   {a="(T)%-?(aN)(%s)([rlmnwy])", b="%1ً%3%4%4"},
@@ -482,8 +493,8 @@ tanwineasy = { -- 'easy' requires some lines to be taken out:
 --   {a="([^TA])%-?(aN)(%s)([rlmnwy])", b="%1ًا%3%4%4"},
 --   {a="%-?(iNI?)(%s)([rlmnwy])", b="ٍ%2%3%3"},
    -- assimilations (end)
-   {a="(o[%S]-)(%-?aN)(_A)(o)([uai])", b="%5"},
-   {a="(o[%S]-)(%-?aN)(Y)(o)([uai])", b="%5"},
+   {a="(O[%S]-)(%-?aN)(_A)(O)([uai])", b="%5"},
+   {a="(O[%S]-)(%-?aN)(Y)(O)([uai])", b="%5"},
    -- quoted tanwīn (begin)
    {a="%-?(\"uN)", b=""},
    {a="(B)%-?(\"aN)", b="%1"},
@@ -496,6 +507,13 @@ tanwineasy = { -- 'easy' requires some lines to be taken out:
    -- quoted tanwīn (end)
    {a="%-?(uN)", b="ٌ"},
    {a="(B)%-?(aN)", b="%1ً"},
+   -- needed by \arbcolor:
+   {a="%-?(aN)(O[%S]-%_AO)", b="ً"},
+   {a="%-?(aN)(O[%S]-YO)", b="ً"},
+   {a="(O[%S]-TO)%-?(aN)", b="ً"},
+   {a="(O[%S]-)([اآ])(ء)(O)%-?(aN)", b="ً"}, --new
+   {a="(O[%S]-[^TA]O)%-?(aN)", b="ًا"},
+   --
    {a="%-?(aN)(_A)", b="ًى"},
    {a="%-?(aN)(Y)", b="ًى"},
    {a="(T)%-?(aN)", b="%1ً"},
@@ -505,46 +523,49 @@ tanwineasy = { -- 'easy' requires some lines to be taken out:
 }
 
 trigraphs = { -- trigraphs or more
+   -- ʾalif al-waṣl: put it back on with \arbnull
+   {a="(O[%S]-)([%'a]l%-)(O)(\"[uai])", b="ٱ"},
+   {a="(O[%S]-)([%'a]l%-)(O)([uai])", b="ا"},
    -- 'llatI / 'llad_I
    {a="^'ll(a)([%_]?[dt])", b="الّ%1%2"},
    {a="([%(%[%|%<%s%-])'ll(a)([%_]?[dt])", b="%1الّ%2%3"}, --p
    -- law: the diphthong is to be resolved into 'awi' (next 8 lines)
-   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
-   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
-   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
-   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
+   {a="(%W)(law)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1%2i"},
+   {a="^(law)(O)(\"?[uai])([%S]-O)", b="%1i"},
+   {a="(%W)(law)(O)(\"?[uai])([%S]-O)", b="%1%2i"},
    {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
    {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
    {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"}, --p
    -- al- + lām
    {a="^(a)l%-(l)", b="ا%1ل%2%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(l)", b="%1ا%2ل%3%3"}, --p
    -- al- + solar consonant ('c' and '^n' are additional characters)
    {a="^(a)l%-(%^n)", b="ا%1ل%2"}, -- ^n is lunar
-   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1ا%2ل%3"},-- ^n is lunar --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(%^n)", b="%1ا%2ل%3"},-- ^n is lunar --p
    {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="ا%1ل%2%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3%3"}, --p
    -- assim. art. + solar consonant ('c' and '^n' are additional characters)
    {a="^(a)(%^n)%-", b="ا%1ل"}, -- ^n is lunar
-   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1ا%2ل"},-- ^n is lunar --p
+   {a="([%(%[%|%<%s%-O])(a)(%^n)%-", b="%1ا%2ل"},-- ^n is lunar --p
    {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="ا%1ل%2"},
-   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل%3"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\")([uai])", b="ا%1ل%3ٱ"},
-   {a="([%(%[%|%<%s%-o])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"}, --p
    {a="^(a)l%-([uai])", b="ا%1ل%2ا"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([uai])", b="%1ا%2ل%3ا"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-([uai])", b="%1ا%2ل%3ا"}, --p
    -- li-/la- + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-(\")([uai])", b="ل%1ل%3ٱ"},
    {a="l([ai])%-l%-([uai])", b="ل%1ل%2ا"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="ا%1ل"},
-   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1ا%2ل"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-", b="%1ا%2ل"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
-   {a="(aW)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="awuا"},
-   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
-   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(aW)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="awuا"},
+   {a="(aw)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1u"},
+   {a="(ay)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
    {a="(aW)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="awuا%2%3"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
@@ -589,53 +610,56 @@ trigraphs = { -- trigraphs or more
 idgham = {
    -- assimilations
    {a="(n)(%s)([rlmnwy])", b="%1%2%3%3"},
-   {a="(n)(o)([rlmnwy])([%S]-o)", b="%3"}
+   {a="(n)(O)([rlmnwy])([%S]-O)", b="%3"}
 }
 
 trigraphseasy = { -- differences marked below with 'easy'
+   -- ʾalif al-waṣl: put it back on with \arbnull
+   {a="(O[%S]-)([%'a]l%-)(O)(\"[uai])", b="ٱ"},
+   {a="(O[%S]-)([%'a]l%-)(O)([uai])", b="ا"},
    -- Allah (easy)
    {a="l%-l_ah", b="l-ll_ah"},
    -- 'llatI / 'llad_I
    {a="^'ll(a)([%_]?[dt])", b="الّ%1%2"},
    {a="([%(%[%|%<%s%-])'ll(a)([%_]?[dt])", b="%1الّ%2%3"}, --p
    -- law: the diphthong is to be resloved into 'awi' (next 8 lines)
-   {a="^(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
-   {a="(%W)(law)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1%2i"},
-   {a="^(law)(o)(\"?[uai])([%S]-o)", b="%1i"},
-   {a="(%W)(law)(o)(\"?[uai])([%S]-o)", b="%1%2i"},
+   {a="^(law)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
+   {a="(%W)(law)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1%2i"},
+   {a="^(law)(O)(\"?[uai])([%S]-O)", b="%1i"},
+   {a="(%W)(law)(O)(\"?[uai])([%S]-O)", b="%1%2i"},
    {a="^(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
    {a="(%W)(law)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1%2i%3%4"},
    {a="^(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1i%2%3"}, --p
    {a="(%W)(law)(%s)([%(%[%|%<]?\"?[uai])", b="%1%2i%3%4"}, --p
    -- al- + lām (easy)
    {a="^(a)l%-(l)", b="ا%1ل%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-(l)", b="%1ا%2ل%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(l)", b="%1ا%2ل%3"}, --p
    -- al- + solar consonant (easy) ('c' and '^n' are additional characters)
    {a="^(a)l%-(%^n)", b="ا%1ل%2"}, -- ^n is lunar
-   {a="([%(%[%|%<%s%-o])(a)l%-(%^n)", b="%1ا%2ل%3"}, -- ^n is lunar --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(%^n)", b="%1ا%2ل%3"}, -- ^n is lunar --p
    {a="^(a)l%-([%_%^%.]?[tdrzsnc])", b="ا%1ل%2"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-([%_%^%.]?[tdrzsnc])", b="%1ا%2ل%3"}, --p
    -- assim. art. + solar consonant (easy) ('c' and '^n' are
    -- additional characters)
    {a="^(a)(%^n)%-", b="ا%1ل"}, -- ^n is lunar
-   {a="([%(%[%|%<%s%-o])(a)(%^n)%-", b="%1ا%2ل"}, -- ^n is lunar --p
+   {a="([%(%[%|%<%s%-O])(a)(%^n)%-", b="%1ا%2ل"}, -- ^n is lunar --p
    {a="^(a)([%_%^%.]?[tdrzsnc])%-", b="ا%1ل"},
-   {a="([%(%[%|%<%s%-o])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل"}, --p
+   {a="([%(%[%|%<%s%-O])(a)([%_%^%.]?[tdrzsnc])%-", b="%1ا%2ل"}, --p
    -- al- + initial unstable hamza
    {a="^(a)l%-(\")([uai])", b="ا%1ل%3ٱ"},
-   {a="([%(%[%|%<%s%-o])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-(\")([uai])", b="%1ا%2ل%4ٱ"}, --p
    {a="^(a)l%-([uai])", b="ا%1ل%2ا"},
-   {a="([%(%[%|%<%s%-o])(a)l%-([uai])", b="%1ا%2ل%3ا"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-([uai])", b="%1ا%2ل%3ا"}, --p
    -- li-/la- + art. + initial unstable hamza is a special orthography
    {a="l([ai])%-l%-(\")([uai])", b="ل%1ل%3ٱ"},
    {a="l([ai])%-l%-([uai])", b="ل%1ل%2ا"},
    -- al- + lunar consonant (i.e. what remains)
    {a="^(a)l%-", b="ا%1ل"},
-   {a="([%(%[%|%<%s%-o])(a)l%-", b="%1ا%2ل"}, --p
+   {a="([%(%[%|%<%s%-O])(a)l%-", b="%1ا%2ل"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
-   {a="(aW)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="awuا"},
-   {a="(aw)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1u"},
-   {a="(ay)(o)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-o)", b="%1i"},
+   {a="(aW)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="awuا"},
+   {a="(aw)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1u"},
+   {a="(ay)(O)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)([%S]-O)", b="%1i"},
    {a="(aW)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="awuا%2%3"},
    {a="(aw)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1u%2%3"},
    {a="(ay)(%s)(['][%_%^%.]?[l'btjghxdrzs`fqkmnwy]%-)", b="%1i%2%3"},
@@ -694,18 +718,18 @@ digraphs = {
    {a="^\"[uai]", b="ٱ"},
    {a="([%(%[%|%<%s%-])\"[uai]", b="%1ٱ"}, --p
    -- diphthongs to be resolved before ʾalif conjunctionis
-   {a="(aW)(o)(\"?[uai])([%S]-o)", b="awuا"},
+   {a="(aW)(O)(\"?[uai])([%S]-O)", b="awuا"},
    {a="(aW)(%s)([%(%[%|%<]?)([uai])", b="awuا%2%3%4"}, --p
-   {a="(aw)(o)(\"?[uai])([%S]-o)", b="%1u"},
+   {a="(aw)(O)(\"?[uai])([%S]-O)", b="%1u"},
    {a="(aw)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1u%2%3ا"}, --p
-   {a="(ay)(o)(\"?[uai])([%S]-o)", b="%1i"},
+   {a="(ay)(O)(\"?[uai])([%S]-O)", b="%1i"},
    {a="(ay)(%s)([%(%[%|%<]?)(\"?[uai])", b="%1i%2%3ا"}, --p
    -- hyphen + initial alif without hamza:
    {a="([uai]%-)(\"?[uai])([%^%_%.%`]?)([%aإأؤئ])", b="%1ا%3%4"},
    -- initial alif without hamza
    {a="^([%(%[%|%<]?)(\"?[uai])", b="%1ا%2"}, --p
    -- initial alif without hamza
-   {a="(o[%S]-)([uaiUAIY])(o)(\"?[uai])", b="ا"},
+   {a="(O[%S]-)([uaiUAIY])(O)(\"?[uai])", b="ا"},
    {a="(%s)([%(%[%|%<]?)(\"?[uai])", b="%1%2ا"}, --p
    {a="%-%-", b="ـ"},
    {a="ؤؤ", b="ؤّ"},
@@ -815,6 +839,7 @@ single = {
    {a="\"$", b="ْ"},
    {a="\"(%W)", b="ْ%1"},
    {a="\"([^uaiUAI])", b="ْ%1"},
+   {a="o", b="ْ"}, -- hard-coded sukūn
    {a="([^0-9])%-([^0-9])", b="%1%2"},
    {a="B", b="ـ"}
 }
@@ -836,6 +861,9 @@ shortv = {
    {a="\"u", b=""},
    {a="\"a", b=""},
    {a="\"i", b=""},
+   {a="%-?%.u", b="ُ"},
+   {a="%-?%.a", b="َ"},
+   {a="%-?%.i", b="ِ"},
    {a="u", b="ُ"},
    {a="a", b="َ"},
    {a="i", b="ِ"}
@@ -868,7 +896,8 @@ punctuation = {
 
 null = {
    {a="%|", b=""},
+   {a="^%-", b=""},
    {a="([^0-9])(%-)", b="%1"},
-   {a="o[%S]-o", b=""},
+   {a="O[%S]-O", b=""},
    {a="[%^%_](.)", b=">??<%1"}
 }
